[perl #50538] when( @n && %n ) fails to smart match
[p5sagit/p5-mst-13.2.git] / opcode.pl
index e8c43dd..ef9ab1e 100755 (executable)
--- a/opcode.pl
+++ b/opcode.pl
@@ -344,6 +344,7 @@ my %opflags = (
 
 my %OP_IS_SOCKET;
 my %OP_IS_FILETEST;
+my %OP_IS_FT_ACCESS;
 my $OCSHIFT = 9;
 my $OASHIFT = 13;
 
@@ -352,12 +353,12 @@ for my $op (@ops) {
     my $flags = $flags{$op};
     for my $flag (keys %opflags) {
        if ($flags =~ s/$flag//) {
-           die "Flag collision for '$op' ($flags{$op}, $flag)"
+           die "Flag collision for '$op' ($flags{$op}, $flag)\n"
                if $argsum & $opflags{$flag};
            $argsum |= $opflags{$flag};
        }
     }
-    die qq[Opcode '$op' has no class indicator ($flags{$op} => $flags)]
+    die qq[Opcode '$op' has no class indicator ($flags{$op} => $flags)\n]
        unless exists $opclass{$flags};
     $argsum |= $opclass{$flags} << $OCSHIFT;
     my $argshift = $OASHIFT;
@@ -366,6 +367,7 @@ for my $op (@ops) {
            # record opnums of these opnames
            $OP_IS_SOCKET{$op}   = $opnum{$op} if $arg =~ s/s//;
            $OP_IS_FILETEST{$op} = $opnum{$op} if $arg =~ s/-//;
+           $OP_IS_FT_ACCESS{$op} = $opnum{$op} if $arg =~ s/\+//;
         }
        my $argnum = ($arg =~ s/\?//) ? 8 : 0;
         die "op = $op, arg = $arg\n"
@@ -403,6 +405,7 @@ EO_OP_IS_COMMENT
 
 gen_op_is_macro( \%OP_IS_SOCKET, 'OP_IS_SOCKET');
 gen_op_is_macro( \%OP_IS_FILETEST, 'OP_IS_FILETEST');
+gen_op_is_macro( \%OP_IS_FT_ACCESS, 'OP_IS_FILETEST_ACCESS');
 
 sub gen_op_is_macro {
     my ($op_is, $macname) = @_;
@@ -414,20 +417,21 @@ sub gen_op_is_macro {
        } keys %$op_is;
        
        my $last = pop @rest;   # @rest slurped, get its last
-       
+       die "Invalid range of ops: $first .. $last\n" unless $last;
+
+       print ON "#define $macname(op)  \\\n\t(";
+
        # verify that op-ct matches 1st..last range (and fencepost)
        # (we know there are no dups)
        if ( $op_is->{$last} - $op_is->{$first} == scalar @rest + 1) {
            
            # contiguous ops -> optimized version
-           print ON "#define $macname(op)      \\\n\t(";
            print ON "(op) >= OP_" . uc($first) . " && (op) <= OP_" . uc($last);
            print ON ")\n\n";
        }
        else {
-           print ON "\n#define $macname(op)    \\\n\t(";
            print ON join(" || \\\n\t ",
-                         map { "(op) == OP_" . uc() } sort keys %OP_IS_SOCKET);
+                         map { "(op) == OP_" . uc() } sort keys %$op_is);
            print ON ")\n\n";
        }
     }
@@ -436,8 +440,8 @@ sub gen_op_is_macro {
 print OC "/* ex: set ro: */\n";
 print ON "/* ex: set ro: */\n";
 
-close OC or die "Error closing opcode.h: $!";
-close ON or die "Error closing opnames.h: $!";
+close OC or die "Error closing opcode.h: $!\n";
+close ON or die "Error closing opnames.h: $!\n";
 
 foreach ('opcode.h', 'opnames.h') {
     safer_rename_silent $_, "$_-old";
@@ -448,9 +452,9 @@ safer_rename $opname_new, 'opnames.h';
 my $pp_proto_new = 'pp_proto.h-new';
 my $pp_sym_new  = 'pp.sym-new';
 
-open PP, ">$pp_proto_new" or die "Error creating $pp_proto_new: $!";
+open PP, ">$pp_proto_new" or die "Error creating $pp_proto_new: $!\n";
 binmode PP;
-open PPSYM, ">$pp_sym_new" or die "Error creating $pp_sym_new: $!";
+open PPSYM, ">$pp_sym_new" or die "Error creating $pp_sym_new: $!\n";
 binmode PPSYM;
 
 print PP <<"END";
@@ -490,8 +494,8 @@ for (@ops) {
 print PP "\n/* ex: set ro: */\n";
 print PPSYM "\n# ex: set ro:\n";
 
-close PP or die "Error closing pp_proto.h: $!";
-close PPSYM or die "Error closing pp.sym: $!";
+close PP or die "Error closing pp_proto.h: $!\n";
+close PPSYM or die "Error closing pp.sym: $!\n";
 
 foreach ('pp_proto.h', 'pp.sym') {
     safer_rename_silent $_, "$_-old";
@@ -602,7 +606,9 @@ __END__
 # Values for the operands are:
 # scalar      - S            list     - L            array     - A
 # hash        - H            sub (CV) - C            file      - F
-# socket      - Fs           filetest - F-           reference - R
+# socket      - Fs           filetest - F-           filetest_access - F-+
+
+# reference - R
 # "?" denotes an optional operand.
 
 # Nothing.
@@ -767,7 +773,7 @@ vec         vec                     ck_fun          ist@    S S S
 index          index                   ck_index        isT@    S S S?
 rindex         rindex                  ck_index        isT@    S S S?
 
-sprintf                sprintf                 ck_fun          mst@    S L
+sprintf                sprintf                 ck_fun          fmst@   S L
 formline       formline                ck_fun          ms@     S L
 ord            ord                     ck_fun          ifsTu%  S?
 chr            chr                     ck_fun          fsTu%   S?
@@ -948,12 +954,12 @@ getpeername       getpeername             ck_fun          is%     Fs
 
 lstat          lstat                   ck_ftst         u-      F
 stat           stat                    ck_ftst         u-      F
-ftrread                -R                      ck_ftst         isu-    F-
-ftrwrite       -W                      ck_ftst         isu-    F-
-ftrexec                -X                      ck_ftst         isu-    F-
-fteread                -r                      ck_ftst         isu-    F-
-ftewrite       -w                      ck_ftst         isu-    F-
-fteexec                -x                      ck_ftst         isu-    F-
+ftrread                -R                      ck_ftst         isu-    F-+
+ftrwrite       -W                      ck_ftst         isu-    F-+
+ftrexec                -X                      ck_ftst         isu-    F-+
+fteread                -r                      ck_ftst         isu-    F-+
+ftewrite       -w                      ck_ftst         isu-    F-+
+fteexec                -x                      ck_ftst         isu-    F-+
 ftis           -e                      ck_ftst         isu-    F-
 ftsize         -s                      ck_ftst         istu-   F-
 ftmtime                -M                      ck_ftst         stu-    F-